# This program is public domain
# Author: Paul Kienzle

#:
avogadro_number = 6.02214179e23 #(30) mol-1
#:
plancks_constant = 4.13566733e-15 #(10) eV s
#:
electron_volt = 1.602176487e-19 #(40) J / eV
#:
speed_of_light = 299792458 # m/s (exact)
#:
electron_radius = 2.8179402894e-15 #(58) m

# From NIST Reference on Constants, Units, and Uncertainty
#   http://physics.nist.gov/cuu/index.html
# neutron mass = 1.008 664 915 97(43) u
# atomic mass constant m_u = 1.660 538 782(83) x 10-27 kg
#:
neutron_mass = 1.00866491597 #(43) u
#:
atomic_mass_constant = 1.660538782e-27 #(83) kg / u
#:
electron_mass = 5.48577990946e-4 #(22) u
